﻿//#define Debug

using System;
using System.IO;
using System.Windows.Forms;
using libWiiSharp;

namespace wadPatch
{
    class Program
    {
        private const string version = "1.0";

        static void Main(string[] args)
        {
            printInfo();
            if (!File.Exists("libWiiSharp.dll")) { Console.WriteLine("libWiiSharp.dll wasn't found..."); exit(); }
            if (args.Length < 2) { printInstructions(); exit(); }

            wadPatch(args);
        }

        private static void wadPatch(string[] args)
        {
            //Parse args, first arg to be file
            string inputFile = args[0];
            string outputFile = null;

            bool useoutputFile = false;

            if (!File.Exists(inputFile)) { Console.WriteLine("{0} wasn't found...", inputFile); exit(); }

            bool[] patches = new bool[3];
            int newSlot = -1;
            int newVersion = -1;

            for (int i = 1; i < args.Length; i++)
            {
                switch (args[i].ToUpper())
                {
                    case "-V":
                        {
                            if (!int.TryParse(args[i + 1], out newVersion))
                            { Console.WriteLine("Invalid version {0}...", args[i + 1]); exit(); }
                            if (newVersion < 0 || newVersion > 0xFFFF)
                            { Console.WriteLine("Invalid version {0}...", newVersion); exit(); }
                        }
                        break;
                    case "-O":
                        {
                            useoutputFile = true;
                            outputFile = args[i + 1];
                        }
                        break;
                }
            }

            if (newSlot == -1 && newVersion == -1)
            { Console.WriteLine("No action specified..."); exit(); }

            //Load WAD
            Console.WriteLine("Loading WAD...");

            WAD w = new WAD();
            w.KeepOriginalFooter = true;

            w.Warning += new EventHandler<MessageEventArgs>(w_Warning);
            w.LoadFile(inputFile);

            //Change slot / version if requested
            if (newVersion > -1 || newSlot > -1)
            {
                if (newVersion > -1)
                { Console.WriteLine("Changing Title Version to: {0}", newVersion); w.TitleVersion = (ushort)newVersion; }

                w.FakeSign = true;
            }

            //Save patched WAD
            Console.WriteLine("");
            if (newSlot == -1 && newVersion == -1) { Console.WriteLine("Nothing applied..."); exit(); }

            Console.WriteLine("Saving WAD...");

            if (useoutputFile)
            {
                w.Save(outputFile);
            }
            else
                w.Save(inputFile);

            Console.WriteLine("Finished...");
            exit();
        }

        static void w_Warning(object sender, MessageEventArgs e)
        {
            System.Windows.Forms.MessageBox.Show(e.Message);
        }

        static void patcher_Debug(object sender, MessageEventArgs e)
        {
            if (!e.Message.ToLower().Contains("finished"))
                Console.WriteLine(e.Message);
        }

        private static void printInfo()
        {
            Console.WriteLine("-=-=-=-=-=-=-=-=-=-=-=-=-");
            Console.WriteLine(" WadPatcher {0} by Scooby74029  ", version);
            Console.WriteLine("-=-=-=-=-=-=-=-=-=-=-=-=-");
            Console.WriteLine("");
        }

        private static void printInstructions()
        {
            Console.WriteLine("Usage: {0}.exe inputfilename.wad [-slot 70] [-v 65535] [-O outputfilename.wad]", Path.GetFileNameWithoutExtension(Application.ExecutablePath));
            Console.WriteLine("     -v 65535 => Changes the version of the IOS (optional)");
            Console.WriteLine("     -O  => Output file .wad(optional)");
        }

        private static void exit()
        {
            //Wait if debugging
#if Debug
            Console.ReadLine();
#endif

            //Exit application
            Environment.Exit(0);
        }
    }
}
